﻿const nsIPrefBranch = Components.interfaces.nsIPrefBranch;

var Logger = 
{
    log : function(msg)
    {
        if (["@mozilla.org/consoleservice;1"] in Components.classes)
        {
            this.console = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage(msg);
        }else
        {
            this.console = Application.console.log(msg);
        }        
    }
};

var XPCU = 
{ 
  getService: function(aURL, aInterface)
  {
    try {
      return Components.classes[aURL].getService(Components.interfaces[aInterface]);
    } catch (ex) {
      Logger.log("Error getting service: " + aURL + ", " + aInterface + "\n" + ex);
      return null;
    }
  },

  createInstance: function (aURL, aInterface)
  {
    try {
      return Components.classes[aURL].createInstance(Components.interfaces[aInterface]);
    } catch (ex) {
      Logger.log("Error creating instance: " + aURL + ", " + aInterface + "\n" + ex);
      return null;
    }
  },

  QI: function(aEl, aIName)
  {
    try {
      return aEl.QueryInterface(Components.interfaces[aIName]);
    } catch (ex) {
      throw("Unable to QI " + aEl + " to " + aIName);
    }
  }
};

var PrefUtils = 
{
    mPrefs: null,
  
    init: function()
    {
        var prefService = XPCU.getService("@mozilla.org/preferences-service;1", "nsIPrefService");
        this.mPrefs = prefService.getBranch("forexpros.");
    },

    addObserver: function addObserver(aDomain, aFunction)
    {
        if (!this.mPrefs) 
            this.init();

        var pbi = XPCU.QI(this.mPrefs, "nsIPrefBranch2");
    
        if (pbi)
            pbi.addObserver(aDomain, aFunction, false);
    },

    removeObserver: function removeObserver(aDomain, aFunction)
    {
        if (!this.mPrefs) 
            this.init();

        var pbi = XPCU.QI(this.mPrefs, "nsIPrefBranch2");
    
        if (pbi)
            pbi.removeObserver(aDomain, aFunction);
    },

    setPref: function(aName, aValue)
    {
        if (!this.mPrefs) 
            this.init();
 
        var type = this.mPrefs.getPrefType(aName);
    
        try 
        {
            if (type == nsIPrefBranch.PREF_STRING) 
            {
                var str = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
                str.data = aValue;
                this.mPrefs.setComplexValue(aName, Components.interfaces.nsISupportsString, str);
            } else if (type == nsIPrefBranch.PREF_BOOL) 
            {
                this.mPrefs.setBoolPref(aName, aValue);
            } else if (type == nsIPrefBranch.PREF_INT) 
            {
                this.mPrefs.setIntPref(aName, aValue);
            }
        } catch(ex) 
        {
            Logger.log("ERROR: Unable to write pref \"" + aName + "\".\n");
        }
    },

    getPref: function(aName)
    {
        if (!this.mPrefs) 
            this.init();

        var type = this.mPrefs.getPrefType(aName);

        try 
        {
            if (type == nsIPrefBranch.PREF_STRING) 
            {
                return this.mPrefs.getComplexValue(aName, Components.interfaces.nsISupportsString).data;
            } else if (type == nsIPrefBranch.PREF_BOOL) 
            {
                return this.mPrefs.getBoolPref(aName);
            } else if (type == nsIPrefBranch.PREF_INT) 
            {
                return this.mPrefs.getIntPref(aName);
            }
        } catch(ex) 
        {
            Logger.log("ERROR: Unable to read pref \"" + aName + "\".\n");
        }
        
        return null;
    }
};